package cn.com.qmth.stmms.biz.exam.service.impl;

import java.util.ArrayList;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import cn.com.qmth.stmms.biz.common.BaseQueryService;
import cn.com.qmth.stmms.biz.exam.model.ExamStudent;
import cn.com.qmth.stmms.biz.exam.model.MarkGroup;
import cn.com.qmth.stmms.biz.exam.model.SelectiveGroup;
import cn.com.qmth.stmms.biz.exam.query.ExamStudentSearchQuery;
import cn.com.qmth.stmms.biz.exam.service.ExamStudentService;
import cn.com.qmth.stmms.biz.exam.service.InspectedService;
import cn.com.qmth.stmms.biz.exam.service.MarkGroupService;
import cn.com.qmth.stmms.biz.exam.service.SelectiveGroupService;
import cn.com.qmth.stmms.biz.user.service.UserService;
import cn.com.qmth.stmms.biz.utils.TaskLock;
import cn.com.qmth.stmms.biz.utils.TaskLockUtil;
import cn.com.qmth.stmms.common.enums.SelectiveStatus;
import cn.com.qmth.stmms.common.enums.SubjectiveStatus;

@Service
public class InspectedServiceImpl extends BaseQueryService<ExamStudent> implements InspectedService {

    @PersistenceContext
    private EntityManager entityManager;

    @Autowired
    private ExamStudentService studentService;

    @Autowired
    private UserService userService;

    @Autowired
    private MarkGroupService groupService;

    @Autowired
    private SelectiveGroupService selectiveGroupService;

    @SuppressWarnings("unchecked")
    @Override
    public List<ExamStudent> findByQuery(ExamStudentSearchQuery query, SubjectiveStatus status, Integer mainNumber,
            Double mainStartScore, Double mainEndScore, Double questionScore, SelectiveStatus selectiveStatus) {
        StringBuilder dataSql = new StringBuilder("select s.id from eb_exam_student s ");
        StringBuilder limitSql = new StringBuilder(" limit :offset,:pageSize");
        Query dataQuery = getQuery(query, status, mainNumber, mainStartScore, mainEndScore, questionScore,
                selectiveStatus, dataSql, limitSql);
        dataQuery.setParameter("offset", (query.getPageNumber() - 1) * query.getPageSize());
        dataQuery.setParameter("pageSize", query.getPageSize());
        List<Integer> list = dataQuery.getResultList();
        List<ExamStudent> resultList = new ArrayList<ExamStudent>();
        if (list != null && !list.isEmpty()) {
            for (Integer id : list) {
                ExamStudent e = studentService.findById(id);
                if (e.getInspectorId() != null) {
                    e.setInspector(userService.findById(e.getInspectorId()));
                }
                resultList.add(e);
            }
        }
        return resultList;

    }

    private Query getQuery(ExamStudentSearchQuery query, SubjectiveStatus status, Integer mainNumber,
            Double mainStartScore, Double mainEndScore, Double questionScore, SelectiveStatus selectiveStatus,
            StringBuilder dataSql, StringBuilder limitSql) {
        StringBuilder whereSql = new StringBuilder(" WHERE s.is_upload = 1 and s.is_absent = 0 and s.is_breach = 0 ");
        if (query.getExamId() != null) {
            whereSql.append(" and s.exam_id = :examId");
        }
        if (StringUtils.isNotBlank(query.getSubjectCode())) {
            whereSql.append(" and s.subject_code = :subjectCode");
        }
        if (status != null) {
            whereSql.append(" and s.subjective_status = :status");
        } else {
            whereSql.append(" and s.subjective_status IN ('MARKED','INSPECTED')");
        }
        if (query.getStartScore() != null) {
            whereSql.append(" and (cast(s.objective_score as decimal(10,3))+cast(s.subjective_score as decimal(10,3))) >= :startScore");
        }
        if (query.getEndScore() != null) {
            whereSql.append(" and (cast(s.objective_score as decimal(10,3))+cast(s.subjective_score as decimal(10,3))) <= :endScore");
        }
        if (StringUtils.isNotBlank(query.getSecretNumber())) {
            whereSql.append(" and s.secret_number = :secretNumber");
        }
        if (mainNumber != null || mainStartScore != null || mainEndScore != null || questionScore != null) {
            whereSql.append(" and exists (select e.student_id from eb_subjective_score e where e.student_id = s.id ");
            if (mainNumber != null) {
                whereSql.append(" and e.main_number = :mainNumber");
            }
            if (mainStartScore != null) {
                whereSql.append(" and e.main_score >= :mainStartScore");
            }
            if (mainEndScore != null) {
                whereSql.append(" and e.main_score <= :mainEndScore");
            }
            if (questionScore != null) {
                whereSql.append(" and e.score = :questionScore");
            }
            whereSql.append(" ) ");
        }
        // 全部选做题为未选做
        List<MarkGroup> groups = groupService.findByExamAndSubjectAndSelective(query.getExamId(),
                query.getSubjectCode(), true);
        List<SelectiveGroup> selectiveGroups = selectiveGroupService.findByExamIdAndSubjectCode(query.getExamId(),
                query.getSubjectCode());
        if (selectiveStatus != null && SelectiveStatus.UN_SELECTIVE.equals(selectiveStatus) && !groups.isEmpty()
                && !selectiveGroups.isEmpty()) {
            whereSql.append(" and not exists (select e.student_id from eb_subjective_score e where e.student_id = s.id and e.group_number in (:groupNumbers) and e.group_score != -1 )");
        }
        // 选做题多选做
        if (selectiveStatus != null && SelectiveStatus.MUTI_SELECTIVE.equals(selectiveStatus) && !groups.isEmpty()
                && !selectiveGroups.isEmpty()) {
            whereSql.append(" and (select count(distinct ss.group_number) from eb_subjective_score ss where ss.student_id = s.id and ss.group_number in (:groupNumbers) and ss.group_score = -1 )< :unSelectiveCount ");
        }
        dataSql.append(whereSql);
        StringBuilder orderSql = new StringBuilder(" order by s.inspect_time desc ");
        dataSql.append(orderSql);
        if (limitSql != null) {
            dataSql.append(limitSql);
        }
        Query dataQuery = entityManager.createNativeQuery(dataSql.toString());

        if (query.getExamId() != null) {
            dataQuery.setParameter("examId", query.getExamId());
        }
        if (StringUtils.isNotBlank(query.getSubjectCode())) {
            dataQuery.setParameter("subjectCode", query.getSubjectCode());
        }
        if (status != null) {
            dataQuery.setParameter("status", status.toString());
        }
        if (query.getStartScore() != null) {
            dataQuery.setParameter("startScore", query.getStartScore());
        }
        if (query.getEndScore() != null) {
            dataQuery.setParameter("endScore", query.getEndScore());
        }
        if (mainNumber != null) {
            dataQuery.setParameter("mainNumber", mainNumber);
        }
        if (mainStartScore != null) {
            dataQuery.setParameter("mainStartScore", mainStartScore);
        }
        if (mainEndScore != null) {
            dataQuery.setParameter("mainEndScore", mainEndScore);
        }
        if (questionScore != null) {
            dataQuery.setParameter("questionScore", questionScore);
        }
        if (StringUtils.isNotBlank(query.getSecretNumber())) {
            dataQuery.setParameter("secretNumber", query.getSecretNumber());
        }
        if (selectiveStatus != null && !groups.isEmpty() && !selectiveGroups.isEmpty()) {
            List<Integer> groupNumbers = new ArrayList<Integer>();
            for (int i = 0; i < groups.size(); i++) {
                groupNumbers.add(groups.get(i).getNumber());
            }
            dataQuery.setParameter("groupNumbers", groupNumbers);
            if (SelectiveStatus.MUTI_SELECTIVE.equals(selectiveStatus)) {
                int unSelectiveCount = selectiveGroups.size() - selectiveGroups.get(0).getSelectiveCount();
                dataQuery.setParameter("unSelectiveCount", unSelectiveCount);
            }
        }
        return dataQuery;
    }

    @Override
    public Integer countByQuery(ExamStudentSearchQuery query, SubjectiveStatus status, Integer mainNumber,
            Double mainStartScore, Double mainEndScore, Double questionScore, SelectiveStatus selectiveStatus) {
        StringBuilder countSql = new StringBuilder("select count(s.id) from eb_exam_student s ");
        Query countQuery = getQuery(query, status, mainNumber, mainStartScore, mainEndScore, questionScore,
                selectiveStatus, countSql, null);
        Object singleResult = countQuery.getResultList().get(0);
        Integer count = singleResult == null ? 0 : Integer.valueOf(singleResult.toString());
        return count;
    }

    @Override
    public void releaseByUserId(Integer examId, String subjectCode, Integer userId) {
        String key = examId + "_" + subjectCode;
        TaskLock taskLock = TaskLockUtil.getInspectedStudentTask(key);
        taskLock.clear(userId);
    }

    @Override
    public void releaseByStudent(ExamStudent student) {
        TaskLock taskLock = TaskLockUtil.getInspectedStudentTask(getKey(student));
        taskLock.remove(student.getId(), 1);
    }

    @Override
    public boolean applyStudent(ExamStudent student, Integer userId) {
        TaskLock taskLock = TaskLockUtil.getInspectedStudentTask(getKey(student));
        boolean lock = taskLock.add(student.getId(), 1, userId);
        // 上锁失败直接返回
        if (!lock) {
            return false;
        }
        // 重复校验任务状态
        if (student.getSubjectiveStatus().equals(SubjectiveStatus.MARKED)) {
            return true;
        } else {
            taskLock.remove(student.getId(), 1, userId);
            return false;
        }
    }

    @Override
    public boolean hasApplied(ExamStudent student, Integer userId) {
        TaskLock taskLock = TaskLockUtil.getInspectedStudentTask(getKey(student));
        return taskLock.exist(student.getId(), 1, userId);
    }

    private String getKey(ExamStudent student) {
        return student.getExamId() + "_" + student.getSubjectCode();
    }
}
