package cn.com.qmth.stmms.biz.mark.service.Impl;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import cn.com.qmth.stmms.biz.exam.model.Exam;
import cn.com.qmth.stmms.biz.exam.model.ExamQuestion;
import cn.com.qmth.stmms.biz.exam.model.ExamStudent;
import cn.com.qmth.stmms.biz.exam.model.ExamSubject;
import cn.com.qmth.stmms.biz.exam.model.MarkGroup;
import cn.com.qmth.stmms.biz.exam.service.ExamQuestionService;
import cn.com.qmth.stmms.biz.exam.service.ExamService;
import cn.com.qmth.stmms.biz.exam.service.ExamStudentService;
import cn.com.qmth.stmms.biz.exam.service.ExamSubjectService;
import cn.com.qmth.stmms.biz.exam.service.MarkGroupService;
import cn.com.qmth.stmms.biz.exam.service.MarkerService;
import cn.com.qmth.stmms.biz.file.service.FileService;
import cn.com.qmth.stmms.biz.mark.model.ArbitrateHistory;
import cn.com.qmth.stmms.biz.mark.model.MarkLibrary;
import cn.com.qmth.stmms.biz.mark.model.MarkSpecialTag;
import cn.com.qmth.stmms.biz.mark.model.MarkStepDTO;
import cn.com.qmth.stmms.biz.mark.model.MarkTrack;
import cn.com.qmth.stmms.biz.mark.model.SpecialTagDTO;
import cn.com.qmth.stmms.biz.mark.model.Task;
import cn.com.qmth.stmms.biz.mark.model.TrackDTO;
import cn.com.qmth.stmms.biz.mark.model.TrialLibrary;
import cn.com.qmth.stmms.biz.mark.model.TrialTag;
import cn.com.qmth.stmms.biz.mark.model.TrialTrack;
import cn.com.qmth.stmms.biz.mark.query.MarkLibrarySearchQuery;
import cn.com.qmth.stmms.biz.mark.service.MarkLibraryService;
import cn.com.qmth.stmms.biz.mark.service.MarkSpecialTagService;
import cn.com.qmth.stmms.biz.mark.service.MarkTrackService;
import cn.com.qmth.stmms.biz.mark.service.TaskService;
import cn.com.qmth.stmms.biz.mark.service.TrialService;
import cn.com.qmth.stmms.biz.user.service.UserService;
import cn.com.qmth.stmms.biz.utils.ScoreItem;
import cn.com.qmth.stmms.common.enums.ExamType;
import cn.com.qmth.stmms.common.enums.LibraryStatus;
import cn.com.qmth.stmms.common.enums.MarkStatus;
import cn.com.qmth.stmms.common.enums.SubjectiveStatus;

/**
 * 评卷任务服务实现
 *
 * @author LS
 */
@Service("taskService")
public class TaskServiceImpl implements TaskService {

    private static Logger log = LoggerFactory.getLogger(TaskServiceImpl.class);

    public static final double UN_SELECTIVE_SCORE = -1;

    @Autowired
    private MarkLibraryService libraryService;

    @Autowired
    private ExamStudentService studentService;

    @Autowired
    private ExamQuestionService questionService;

    @Autowired
    private MarkTrackService trackService;

    @Autowired
    private MarkGroupService groupService;

    @Autowired
    private MarkSpecialTagService markSpecialTagService;

    @Autowired
    private MarkerService markerService;

    @Autowired
    private TrialService trialService;

    @Autowired
    private ExamSubjectService subjectService;

    @Autowired
    private UserService userService;

    @Autowired
    private FileService fileService;

    @Autowired
    private ExamService examService;

    @Override
    public List<Task> findByQuery(MarkLibrarySearchQuery query) {
        List<Task> list = new LinkedList<Task>();
        query = libraryService.findByQuery(query);
        if (query.getCurrentCount() > 0) {
            for (MarkLibrary library : query.getResult()) {
                list.add(build(library));
            }
        }
        return list;
    }

    @Override
    public Task build(ArbitrateHistory history, MarkGroup group) {
        ExamStudent student = studentService.findByExamIdAndExamNumber(history.getExamId(), history.getExamNumber());
        Task task = new Task();
        task.setStudentId(history.getStudentId());
        task.setLibraryId(history.getId());
        task.setSecretNumber(history.getSecretNumber());
        task.setQuestionList(buildArbitrateStep(group, history));
        task.setSliceUrls(fileService.getSliceUris(student.getExamId(), student.getSecretNumber(), 1,
                student.getSliceCount()));
        task.setSliceConfig(group.getPictureConfigList());
        task.setJsonUrl(fileService.getJsonUri(student.getExamId(), student.getSecretNumber()));
        if (StringUtils.isNotBlank(student.getObjectiveScoreList())) {
            task.setObjectiveScore(student.getObjectiveScore());
        }
        task.setMarkerTime(history.getUpdateTime());
        if (history.getTotalScore() != null) {
            task.setMarkerScore(history.getTotalScore());
        }
        return task;
    }

    private List<MarkStepDTO> buildArbitrateStep(MarkGroup group, ArbitrateHistory history) {
        List<MarkStepDTO> list = new LinkedList<MarkStepDTO>();
        List<ExamQuestion> sList = questionService.findByExamAndSubjectAndObjectiveAndGroupNumber(group.getExamId(),
                group.getSubjectCode(), false, group.getNumber());
        List<MarkTrack> tracks = new ArrayList<MarkTrack>();
        List<ScoreItem> sItems = new ArrayList<ScoreItem>();
        if (history != null) {
            sItems = history.getScoreItemList();
        }
        for (int i = 0; i < sList.size(); i++) {
            ExamQuestion question = sList.get(i);
            MarkStepDTO step = buildStep(question);
            if (history != null) {
                if (!sItems.isEmpty() && sItems.size() == sList.size()) {
                    step.setScore(sItems.get(i).getScore());
                }
                // 未选做
                if (history.getTotalScore() != null && history.getTotalScore() == UN_SELECTIVE_SCORE) {
                    step.setScore(UN_SELECTIVE_SCORE);
                }
                // 增加阅卷轨迹列表获取
                String questionNumber = question.getQuestionNumber();
                for (MarkTrack track : tracks) {
                    if (track.getQuestionNumber().equals(questionNumber)) {
                        step.addTrack(new TrackDTO(track));
                    }
                }
            }
            list.add(step);
        }
        return list;
    }

    @Override
    public Task build(MarkLibrary library) {
        Exam exam = examService.findById(library.getExamId());
        ExamSubject subject = subjectService.find(library.getExamId(), library.getSubjectCode());
        ExamStudent student = studentService.findByExamIdAndExamNumber(library.getExamId(), library.getExamNumber());
        MarkGroup group = groupService.findOne(library.getExamId(), library.getSubjectCode(), library.getGroupNumber());

        Task task = new Task();
        task.setLibraryId(library.getId());
        task.setStudentId(library.getStudentId());
        task.setSecretNumber(library.getSecretNumber());
        task.setStudentCode(student.getStudentCode());
        task.setStudentName(student.getName());
        task.setExamNumber(library.getExamNumber());
        task.setSubject(subject);
        if (ExamType.MULTI_MEDIA.equals(exam.getType())) {
            task.setJsonUrl(fileService.getJsonUri(library.getExamId(), library.getSecretNumber()));
        } else {
            task.setSliceUrls(fileService.getSliceUris(library.getExamId(), library.getSecretNumber(), 1,
                    student.getSliceCount()));
            task.setSliceConfig(group.getPictureConfigList());
            task.setSheetUrls(fileService.getSheetUris(library.getExamId(), library.getExamNumber(), 1,
                    student.getSheetCount()));
        }
        task.setQuestionList(buildMarkStep(group, library));
        task.setSpecialTagList(getMarkSpecialTagList(library.getId()));
        if (StringUtils.isNotBlank(student.getObjectiveScoreList())) {
            task.setObjectiveScore(student.getObjectiveScore());
        }
        task.setMarkerTime(library.getMarkerTime());
        if (library.getStatus() == LibraryStatus.REJECTED) {
            task.setRejected(true);
        }
        if (library.getStatus() == LibraryStatus.MARKED || library.getStatus() == LibraryStatus.INSPECTED) {
            task.setMarkerScore(library.getMarkerScore());
            task.setPrevious(true);
        }
        return task;
    }

    @Override
    public Task build(TrialLibrary library) {
        Exam exam = examService.findById(library.getExamId());
        ExamSubject subject = subjectService.find(library.getExamId(), library.getSubjectCode());
        ExamStudent student = studentService.findByExamIdAndExamNumber(library.getExamId(), library.getExamNumber());
        MarkGroup group = groupService.findOne(library.getExamId(), library.getSubjectCode(), library.getGroupNumber());

        Task task = new Task();
        task.setLibraryId(library.getId());
        task.setStudentId(library.getStudentId());
        task.setSecretNumber(library.getSecretNumber());
        task.setStudentCode(student.getStudentCode());
        task.setStudentName(student.getName());
        task.setExamNumber(library.getExamNumber());
        task.setSubject(subject);
        task.setQuestionList(buildTrialStep(group, library));
        if (StringUtils.isNotBlank(student.getObjectiveScoreList())) {
            task.setObjectiveScore(student.getObjectiveScore());
        }
        if (ExamType.MULTI_MEDIA.equals(exam.getType())) {
            task.setJsonUrl(fileService.getJsonUri(library.getExamId(), library.getSecretNumber()));
        } else {
            task.setSliceUrls(fileService.getSliceUris(library.getExamId(), library.getSecretNumber(), 1,
                    student.getSliceCount()));
            task.setSliceConfig(group.getPictureConfigList());
            task.setSheetUrls(fileService.getSheetUris(library.getExamId(), library.getExamNumber(), 1,
                    student.getSheetCount()));
        }
        if (library.getMarkerId() != null) {
            task.setSpecialTagList(getTrialTagList(library));
            task.setMarkerTime(library.getMarkerTime());
            task.setMarkerScore(library.getMarkerScore());
            task.setPrevious(true);
        }
        return task;
    }

    private List<MarkStepDTO> buildMarkStep(MarkGroup group, MarkLibrary library) {
        List<MarkStepDTO> list = new LinkedList<MarkStepDTO>();
        List<ExamQuestion> sList = questionService.findByExamAndSubjectAndObjectiveAndGroupNumber(group.getExamId(),
                group.getSubjectCode(), false, group.getNumber());
        List<MarkTrack> tracks = new ArrayList<MarkTrack>();
        List<ScoreItem> sItems = new ArrayList<ScoreItem>();
        if (library != null) {
            tracks = trackService.findByLibraryId(library.getId());
            sItems = library.getScoreList();
        }
        for (int i = 0; i < sList.size(); i++) {
            ExamQuestion question = sList.get(i);
            MarkStepDTO step = buildStep(question);
            if (library != null) {
                if (!sItems.isEmpty() && sItems.size() == sList.size()) {
                    step.setScore(sItems.get(i).getScore());
                }
                // 未选做
                if (library.getMarkerScore() != null && library.getMarkerScore() == UN_SELECTIVE_SCORE) {
                    step.setScore(UN_SELECTIVE_SCORE);
                }
                // 增加阅卷轨迹列表获取
                String questionNumber = question.getQuestionNumber();
                for (MarkTrack track : tracks) {
                    if (track.getQuestionNumber().equals(questionNumber)) {
                        step.addTrack(new TrackDTO(track));
                    }
                }
            }
            list.add(step);
        }
        return list;
    }

    private List<MarkStepDTO> buildTrialStep(MarkGroup group, TrialLibrary library) {
        List<MarkStepDTO> list = new LinkedList<MarkStepDTO>();
        List<ExamQuestion> sList = questionService.findByExamAndSubjectAndObjectiveAndGroupNumber(group.getExamId(),
                group.getSubjectCode(), false, group.getNumber());
        List<TrialTrack> tracks = new ArrayList<TrialTrack>();
        List<ScoreItem> sItems = new ArrayList<ScoreItem>();
        if (library.getMarkerId() != null) {
            tracks = trialService.findTrack(library.getId());
            sItems = library.getScoreList();
        }
        for (int i = 0; i < sList.size(); i++) {
            ExamQuestion question = sList.get(i);
            MarkStepDTO step = buildStep(question);
            if (!sItems.isEmpty() && sItems.size() == sList.size()) {
                step.setScore(sItems.get(i).getScore());
            }
            if (library.getMarkerScore() != null && library.getMarkerScore() == UN_SELECTIVE_SCORE) {
                step.setScore(UN_SELECTIVE_SCORE);
            }
            // 增加阅卷轨迹列表获取
            String questionNumber = question.getQuestionNumber();
            for (TrialTrack track : tracks) {
                if (track.getQuestionNumber().equals(questionNumber)) {
                    step.addTrack(new TrackDTO(track));
                }
            }
            list.add(step);
        }
        return list;
    }

    private MarkStepDTO buildStep(ExamQuestion question) {
        MarkStepDTO step = new MarkStepDTO();
        step.setMainNumber(question.getMainNumber());
        step.setSubNumber(question.getSubNumber());
        step.setGroupNumber(question.getGroupNumber());
        step.setTitle(question.getMainTitle());
        step.setDefaultScore(0d);
        step.setMaxScore(question.getTotalScore());
        step.setMinScore(0d);
        step.setIntervalScore(question.getIntervalScore());
        return step;
    }

    public SpecialTagDTO[] getMarkSpecialTagList(Integer libraryId) {
        SpecialTagDTO[] specialTags = null;
        List<MarkSpecialTag> list = markSpecialTagService.findByLibraryId(libraryId);
        if (!list.isEmpty()) {
            specialTags = new SpecialTagDTO[list.size()];
            for (int i = 0; i < list.size(); i++) {
                SpecialTagDTO specialTagDTO = new SpecialTagDTO(list.get(i));
                specialTags[i] = specialTagDTO;
            }
        }
        return specialTags;
    }

    public SpecialTagDTO[] getTrialTagList(TrialLibrary library) {
        SpecialTagDTO[] specialTags = null;
        List<TrialTag> list = trialService.findTag(library.getId());
        if (!list.isEmpty()) {
            specialTags = new SpecialTagDTO[list.size()];
            for (int i = 0; i < list.size(); i++) {
                SpecialTagDTO specialTagDTO = new SpecialTagDTO(list.get(i));
                specialTags[i] = specialTagDTO;
            }
        }
        return specialTags;
    }

    @Override
    public Task build(ExamStudent student) {
        ExamSubject subject = subjectService.find(student.getExamId(), student.getSubjectCode());
        Exam exam = examService.findById(student.getExamId());

        Task task = new Task();
        task.setStudentId(student.getId());
        task.setSecretNumber(student.getSecretNumber());
        task.setStudentCode(student.getStudentCode());
        task.setStudentName(student.getName());
        task.setExamNumber(student.getExamNumber());
        task.setSubject(subject);
        if (ExamType.MULTI_MEDIA.equals(exam.getType())) {
            task.setJsonUrl(fileService.getJsonUri(student.getExamId(), student.getSecretNumber()));
        } else {
            task.setSliceUrls(fileService.getSliceUris(student.getExamId(), student.getSecretNumber(), 1,
                    student.getSliceCount()));
            task.setSheetUrls(fileService.getSheetUris(student.getExamId(), student.getExamNumber(), 1,
                    student.getSheetCount()));
        }
        if (!student.getSubjectiveStatus().equals(SubjectiveStatus.UNMARK)) {
            long trialGroupCount = groupService.countByExamAndSubjectAndStatus(student.getExamId(),
                    student.getSubjectCode(), MarkStatus.TRIAL);
            task.setQuestionList(buildMarkStep(student, trialGroupCount));
            task.setSpecialTagList(getMarkSpecialTagList(student, trialGroupCount));
        }
        if (StringUtils.isNotBlank(student.getObjectiveScoreList())) {
            task.setObjectiveScore(student.getObjectiveScore());
        }
        task.setMarkerScore(student.getTotalScore());
        task.setInspectTime(student.getInspectTime());
        return task;
    }

    private SpecialTagDTO[] getMarkSpecialTagList(ExamStudent student, long trialGroupCount) {
        List<SpecialTagDTO> list = new ArrayList<SpecialTagDTO>();
        if (trialGroupCount > 0) {
            List<TrialTag> tags = trialService.findTagByStudentId(student.getId());
            for (TrialTag trialTag : tags) {
                list.add(new SpecialTagDTO(trialTag));
            }
        } else {
            List<MarkGroup> groups = groupService.findByExamAndSubject(student.getExamId(), student.getSubjectCode());
            for (MarkGroup group : groups) {
                List<MarkLibrary> libraryList = libraryService
                        .findByStudentAndGroup(student.getId(), group.getNumber());
                if (libraryList.size() == 1) {
                    SpecialTagDTO[] tags = getMarkSpecialTagList(libraryList.get(0).getId());
                    if (tags != null) {
                        for (SpecialTagDTO specialTagDTO : tags) {
                            list.add(specialTagDTO);
                        }
                    }
                }
            }
        }
        SpecialTagDTO[] specialTags = new SpecialTagDTO[list.size()];
        for (int i = 0; i < list.size(); i++) {
            specialTags[i] = list.get(i);
        }
        return specialTags;
    }

    private List<MarkStepDTO> buildMarkStep(ExamStudent student, long trialGroupCount) {
        List<MarkStepDTO> list = new LinkedList<MarkStepDTO>();
        List<ExamQuestion> sList = questionService.findByExamAndSubjectAndObjective(student.getExamId(),
                student.getSubjectCode(), false);
        List<ScoreItem> sItems = student.getScoreList(false);
        for (int i = 0; i < sList.size(); i++) {
            ExamQuestion question = sList.get(i);
            MarkStepDTO step = buildStep(question);
            if (!sItems.isEmpty() && sItems.size() == sList.size()) {
                step.setScore(sItems.get(i).getScore());
            }
            // 增加阅卷轨迹列表获取
            if (trialGroupCount > 0) {
                List<TrialTrack> tracks = trialService.findTrackByStudentId(student.getId());
                String questionNumber = question.getQuestionNumber();
                for (TrialTrack track : tracks) {
                    if (track.getQuestionNumber().equals(questionNumber)) {
                        step.addTrack(new TrackDTO(track));
                    }
                }
            } else {
                List<MarkLibrary> libraryList = libraryService.findByStudentAndGroup(student.getId(),
                        question.getGroupNumber());
                if (libraryList.size() == 1) {
                    List<MarkTrack> tracks = trackService.findByStudentId(student.getId());
                    String questionNumber = question.getQuestionNumber();
                    for (MarkTrack track : tracks) {
                        if (track.getQuestionNumber().equals(questionNumber)) {
                            step.addTrack(new TrackDTO(track));
                        }
                    }
                }
            }
            list.add(step);
        }
        return list;
    }
}
