package cn.com.qmth.stmms.biz.utils;

import java.util.Arrays;

import net.sf.json.JSONObject;

import org.apache.commons.lang3.ArrayUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.qmth.boot.tools.codec.CodecUtils;
import com.qmth.boot.tools.crypto.AES;
import com.qmth.boot.tools.crypto.RSA;
import com.qmth.boot.tools.device.DeviceInfo;
import com.qmth.boot.tools.models.ByteArray;

/**
 * 应用许可证解析工具
 */
public class AppLicenseUtil {

    private static final Logger log = LoggerFactory.getLogger(AppLicenseUtil.class);

    private static final String PUBLIC_KEY_PATH = "license/public.key";

    /**
     * 构造当前设备加密信息文件
     *
     * @return
     */
    public static ByteArray buildDeviceInfo() throws Exception {
        return RSA.getPublicKey(
                ByteArray.fromInputStream(AppLicenseUtil.class.getClassLoader().getResourceAsStream(PUBLIC_KEY_PATH))
                        .value()).encrypt(new ObjectMapper().writeValueAsBytes(DeviceInfo.current()));
    }

    /**
     * 直接解析许可证文件内容
     *
     * @param data
     * @return
     */
    public static JSONObject parseLicense(byte[] data) {
        try {
            byte[] original = ArrayUtils.subarray(data, 1, data.length);
            // 是否绑定硬件
            if (data[0] == 0) {
                // 不绑定硬件
            } else if (data[0] == 1) {
                // 获取当前设备标识
                String deviceId = DeviceInfo.current().uuid();
                // 使用设备标识进行AES解密
                original = AES.decrypt(original, deviceId.substring(0, 16), deviceId.substring(16)).value();
            } else {
                return null;
            }
            // 截取前16位摘要信息
            byte[] digest = ArrayUtils.subarray(original, 0, 16);
            // 进行RSA解密
            byte[] content = RSA
                    .getPublicKey(
                            ByteArray.fromInputStream(
                                    AppLicenseUtil.class.getClassLoader().getResourceAsStream(PUBLIC_KEY_PATH)).value())
                    .decrypt(ArrayUtils.subarray(original, 16, original.length)).value();
            // 比对摘要是否一致
            if (Arrays.equals(digest, CodecUtils.md5(content))) {
                return new ObjectMapper().readValue(content, JSONObject.class);
            }
        } catch (Exception e) {
            log.error("license parse error", e);
        }
        return null;
    }

}
