package cn.com.qmth.stmms.admin.exam;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import net.sf.json.JSONObject;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import cn.com.qmth.stmms.admin.dto.RejectResult;
import cn.com.qmth.stmms.biz.exam.model.Exam;
import cn.com.qmth.stmms.biz.exam.model.ExamStudent;
import cn.com.qmth.stmms.biz.exam.model.ExamSubject;
import cn.com.qmth.stmms.biz.exam.query.ExamStudentSearchQuery;
import cn.com.qmth.stmms.biz.exam.service.ExamQuestionService;
import cn.com.qmth.stmms.biz.exam.service.ExamService;
import cn.com.qmth.stmms.biz.exam.service.ExamStudentService;
import cn.com.qmth.stmms.biz.exam.service.ExamSubjectService;
import cn.com.qmth.stmms.biz.exam.service.InspectedService;
import cn.com.qmth.stmms.biz.file.service.FileService;
import cn.com.qmth.stmms.biz.lock.LockService;
import cn.com.qmth.stmms.biz.mark.model.Task;
import cn.com.qmth.stmms.biz.mark.service.MarkService;
import cn.com.qmth.stmms.biz.mark.service.TaskService;
import cn.com.qmth.stmms.common.annotation.Logging;
import cn.com.qmth.stmms.common.annotation.RoleRequire;
import cn.com.qmth.stmms.common.domain.WebUser;
import cn.com.qmth.stmms.common.enums.ExamType;
import cn.com.qmth.stmms.common.enums.LockType;
import cn.com.qmth.stmms.common.enums.LogType;
import cn.com.qmth.stmms.common.enums.Role;
import cn.com.qmth.stmms.common.enums.SelectiveStatus;
import cn.com.qmth.stmms.common.enums.SubjectiveStatus;
import cn.com.qmth.stmms.common.utils.RequestUtils;

@Controller("inspectedController")
@RequestMapping("/admin/exam/inspected")
public class InspectedController extends BaseExamController {

    protected static Logger log = LoggerFactory.getLogger(InspectedController.class);

    @Autowired
    private ExamStudentService studentService;

    @Autowired
    private InspectedService inspectedService;

    @Autowired
    private ExamQuestionService questionService;

    @Autowired
    private FileService fileService;

    @Autowired
    private LockService lockService;

    @Autowired
    private MarkService markService;

    @Autowired
    private ExamService examService;

    @Autowired
    private TaskService taskService;

    @Autowired
    private ExamSubjectService subjectService;

    @Value("${slice.split.config}")
    private String splitConfig;

    public static final String UN_SELECTIVE_SCORE = "-1";

    @RequestMapping
    public String list(Model model, HttpServletRequest request, ExamStudentSearchQuery query,
            @RequestParam(required = false) SubjectiveStatus status,
            @RequestParam(required = false) SelectiveStatus selectiveStatus,
            @RequestParam(required = false) Integer mainNumber, @RequestParam(required = false) Double mainStartScore,
            @RequestParam(required = false) Double mainEndScore, @RequestParam(required = false) Double questionScore) {
        int examId = getSessionExamId(request);
        Exam exam = examService.findById(examId);
        if (ExamType.MULTI_MEDIA.equals(exam.getType())) {
            return "modules/exam/permission";
        }
        WebUser wu = RequestUtils.getWebUser(request);
        query.setExamId(examId);
        List<ExamSubject> subjectList = getExamSubject(examId, wu);
        if (StringUtils.isBlank(query.getSubjectCode()) && !subjectList.isEmpty()) {
            query.setSubjectCode(subjectList.get(0).getCode());
        }
        List<ExamStudent> list = inspectedService.findByQuery(query, status, mainNumber, mainStartScore, mainEndScore,
                questionScore, selectiveStatus);
        for (ExamStudent e : list) {
            e.setSubjectiveScoreList(e.getSubjectiveScoreList().replace(UN_SELECTIVE_SCORE, "/"));
        }
        Integer totalCount = inspectedService.countByQuery(query, null, mainNumber, mainStartScore, mainEndScore,
                questionScore, selectiveStatus);
        Integer inspectCount = inspectedService.countByQuery(query, SubjectiveStatus.MARKED, mainNumber,
                mainStartScore, mainEndScore, questionScore, selectiveStatus);
        query.setResult(list);
        query.setTotalCount(totalCount);
        model.addAttribute("query", query);
        model.addAttribute("inspectCount", inspectCount);
        model.addAttribute("questionList",
                questionService.findMainByExamAndSubjectAndObjective(examId, query.getSubjectCode(), false));
        model.addAttribute("mainNumber", mainNumber);
        model.addAttribute("mainStartScore", mainStartScore);
        model.addAttribute("mainEndScore", mainEndScore);
        model.addAttribute("questionScore", questionScore);
        model.addAttribute("subjectList", subjectList);
        model.addAttribute("statusList", SubjectiveStatus.getOptionList());
        model.addAttribute("selectiveStatusList", SelectiveStatus.getOptionList());
        model.addAttribute("status", status);
        model.addAttribute("selectiveStatus", selectiveStatus);
        model.addAttribute("exam", exam);
        return "modules/exam/inspectedList";
    }

    @Logging(menu = "开始考生复核", type = LogType.QUERY)
    @RequestMapping(value = "/getTask", method = RequestMethod.POST)
    @ResponseBody
    public Task getTask(HttpServletRequest request, RedirectAttributes redirectAttributes,
            ExamStudentSearchQuery query,
            @RequestParam(required = false, defaultValue = "false") SelectiveStatus selectiveStatus,
            @RequestParam(required = false) Integer mainNumber, @RequestParam(required = false) Double mainStartScore,
            @RequestParam(required = false) Double mainEndScore, @RequestParam(required = false) Double questionScore,
            @RequestParam(required = false) Integer studentId) {
        int examId = getSessionExamId(request);
        WebUser wu = RequestUtils.getWebUser(request);
        Task task = null;
        if (studentId != null) {
            releaseStudent(studentId);
            ExamStudent student = studentService.findById(studentId);
            if (inspectedService.applyStudent(student, wu.getId())) {
                task = taskService.build(student);
            }
        }
        int retry = 1;
        while (task == null) {
            List<ExamStudent> list = new ArrayList<ExamStudent>();
            // 需要判断评卷员是否绑定了班级
            query.setExamId(examId);
            query.setPageNumber(retry);
            query.setPageSize(20);
            List<ExamSubject> subjectList = getExamSubject(examId, wu);
            if (StringUtils.isBlank(query.getSubjectCode()) && !subjectList.isEmpty()) {
                query.setSubjectCode(subjectList.get(0).getCode());
            }
            list = inspectedService.findByQuery(query, SubjectiveStatus.MARKED, mainNumber, mainStartScore,
                    mainEndScore, questionScore, selectiveStatus);
            if (list.isEmpty()) {
                break;
            }
            for (ExamStudent student : list) {
                if (inspectedService.applyStudent(student, wu.getId())) {
                    task = taskService.build(student);
                    break;
                }
            }
            if (task == null) {
                retry++;
            }
        }
        return task;
    }

    @Logging(menu = "考生评卷复核", type = LogType.UPDATE)
    @RequestMapping(value = "/save", method = RequestMethod.POST)
    @ResponseBody
    public Object save(HttpServletRequest request, @RequestParam Integer studentId) {
        WebUser wu = RequestUtils.getWebUser(request);
        ExamStudent student = studentService.findById(studentId);
        JSONObject obj = new JSONObject();
        try {
            if (lockService.trylock(LockType.STUDENT, student.getId())
                    && inspectedService.hasApplied(student, wu.getId())
                    && SubjectiveStatus.MARKED.equals(student.getSubjectiveStatus())) {
                studentService.updateSubjectiveStatusAndTimeAndInspectorId(studentId, SubjectiveStatus.INSPECTED,
                        new Date(), wu.getUser().getId());
                inspectedService.releaseByStudent(student);
                obj.accumulate("success", true);
            } else {
                obj.accumulate("success", false);
                obj.accumulate("message", "无法复核，请刷新页面");
            }
        } catch (Exception e) {
            obj.accumulate("success", false);
            log.error("inspected save error", e);
        } finally {
            lockService.unlock(LockType.STUDENT, student.getId());
        }
        return obj;
    }

    @RequestMapping(value = "/clear", method = RequestMethod.POST)
    @ResponseBody
    @RoleRequire({ Role.SCHOOL_ADMIN, Role.SUBJECT_HEADER, Role.INSPECTOR })
    public JSONObject clear(HttpServletRequest request, @RequestParam(required = false) String subjectCode,
            @RequestParam(required = false) Integer studentId) {
        JSONObject obj = new JSONObject();
        if (studentId != null) {
            releaseStudent(studentId);
            obj.accumulate("success", true);
        } else if (subjectCode != null) {
            int examId = getSessionExamId(request);
            releaseUser(examId, subjectCode, RequestUtils.getWebUser(request).getId());
            obj.accumulate("success", true);
        } else {
            obj.accumulate("success", false);
        }
        return obj;
    }

    @Logging(menu = "取消复核", type = LogType.UPDATE)
    @RequestMapping(value = "/cancel", method = RequestMethod.POST)
    @ResponseBody
    @RoleRequire({ Role.SCHOOL_ADMIN, Role.SUBJECT_HEADER, Role.INSPECTOR })
    public JSONObject cancel(HttpServletRequest request, @RequestParam Integer studentId) {
        JSONObject obj = new JSONObject();
        ExamStudent student = studentService.findById(studentId);
        WebUser wu = RequestUtils.getWebUser(request);
        if (student != null && student.getSubjectiveStatus().equals(SubjectiveStatus.INSPECTED)) {
            if (subjectCheck(student.getSubjectCode(), wu)) {
                try {
                    if (lockService.trylock(LockType.STUDENT, student.getId())) {
                        studentService.updateSubjectiveStatusAndTimeAndInspectorId(student.getId(),
                                SubjectiveStatus.MARKED, null, null);
                        obj.accumulate("success", true);
                    } else {
                        obj.accumulate("success", false);
                        obj.accumulate("message", "无法取消复核");
                    }
                } catch (Exception e) {
                    obj.accumulate("success", false);
                    obj.accumulate("message", "取消复核失败");
                    log.error("back inspected error", e);
                } finally {
                    lockService.unlock(LockType.STUDENT, student.getId());
                }
            } else {
                obj.accumulate("success", false);
                obj.accumulate("message", "没有操作该评卷任务的权限");
            }
        } else {
            obj.accumulate("success", false);
            obj.accumulate("message", "无法取消复核");
        }
        return obj;
    }

    @Logging(menu = "打回", type = LogType.UPDATE)
    @RequestMapping(value = "/rejected", method = RequestMethod.POST)
    @ResponseBody
    @RoleRequire({ Role.SCHOOL_ADMIN, Role.SUBJECT_HEADER, Role.INSPECTOR })
    public JSONObject rejected(HttpServletRequest request, @RequestBody RejectResult rejectResult) {
        JSONObject obj = new JSONObject();
        ExamStudent student = studentService.findById(rejectResult.getStudentId());
        WebUser wu = RequestUtils.getWebUser(request);
        if (student == null) {
            obj.accumulate("success", false);
            obj.accumulate("message", "无法打回");
            return obj;
        }
        if ((inspectedService.hasApplied(student, wu.getId()) && (student.getSubjectiveStatus()
                .equals(SubjectiveStatus.MARKED))) || student.getSubjectiveStatus().equals(SubjectiveStatus.INSPECTED)) {
            if (subjectCheck(student.getSubjectCode(), wu) && lockService.trylock(LockType.STUDENT, student.getId())) {
                try {
                    lockService.watch(LockType.EXAM_SUBJECT, student.getExamId(), student.getSubjectCode());
                    if (markService.rejectedStudent(student, rejectResult.getQuestionList(), wu.getId())) {
                        inspectedService.releaseByStudent(student);
                        obj.accumulate("success", true);
                    } else {
                        obj.accumulate("success", false);
                        obj.accumulate("message", "无法打回");
                    }
                } catch (Exception e) {
                    obj.accumulate("success", false);
                    obj.accumulate("message", "打回失败");
                    log.error("back inspected error", e);
                } finally {
                    lockService.unwatch(LockType.EXAM_SUBJECT, student.getExamId(), student.getSubjectCode());
                    lockService.unlock(LockType.STUDENT, student.getId());
                }
            } else {
                obj.accumulate("success", false);
                obj.accumulate("message", "无法打回");
            }
        }
        return obj;
    }

    @Logging(menu = "回看全卷复核任务", type = LogType.QUERY)
    @RequestMapping(value = "/getHistory", method = RequestMethod.POST)
    @ResponseBody
    public Object getHistory(HttpServletRequest request, @RequestParam String subjectCode,
            @RequestParam int pageNumber, @RequestParam int pageSize) throws Exception {
        int examId = getSessionExamId(request);
        WebUser wu = RequestUtils.getWebUser(request);
        List<Task> list = new ArrayList<>();
        ExamStudentSearchQuery query = new ExamStudentSearchQuery();
        query.setExamId(examId);
        query.setSubjectCode(subjectCode);
        query.setInspectorId(wu.getId());
        query.addStatus(SubjectiveStatus.INSPECTED);
        query.setPageNumber(pageNumber);
        query.setPageSize(pageSize);
        query.orderByInspectTimeDesc();
        query = studentService.findByQuery(query);
        for (ExamStudent student : query.getResult()) {
            Task task = taskService.build(student);
            task.setPrevious(true);
            list.add(task);
        }
        return list;
    }

    @RequestMapping(value = "/getStatus", method = RequestMethod.POST)
    @ResponseBody
    public JSONObject status(HttpServletRequest request, ExamStudentSearchQuery query,
            @RequestParam(required = false) SelectiveStatus selectiveStatus,
            @RequestParam(required = false) Integer mainNumber, @RequestParam(required = false) Double mainStartScore,
            @RequestParam(required = false) Double mainEndScore, @RequestParam(required = false) Double questionScore) {
        JSONObject status = new JSONObject();
        int examId = getSessionExamId(request);
        query.setExamId(examId);
        Integer totalCount = inspectedService.countByQuery(query, SubjectiveStatus.MARKED, mainNumber, mainStartScore,
                mainEndScore, questionScore, selectiveStatus);
        status.accumulate("totalCount", totalCount);
        status.accumulate("valid", true);
        return status;
    }

    @RequestMapping(value = "/getSetting", method = RequestMethod.POST)
    @ResponseBody
    public JSONObject getSetting(HttpServletRequest request, @RequestParam(required = false) String subjectCode) {
        JSONObject setting = new JSONObject();
        WebUser wu = RequestUtils.getWebUser(request);
        int examId = getSessionExamId(request);
        Exam exam = examService.findById(examId);
        setting.accumulate("examType", exam.getType());
        setting.accumulate("fileServer", fileService.getFileServer());
        setting.accumulate("userName", wu.getName());
        setting.accumulate("splitConfig", getSplitConfig());
        if (StringUtils.isNotBlank(subjectCode)) {
            ExamSubject examSubject = subjectService.find(examId, subjectCode);
            JSONObject subject = new JSONObject();
            subject.accumulate("name", examSubject.getName());
            subject.accumulate("code", examSubject.getCode());
            examSubject.setPaperAnswerUrl(fileService);
            subject.accumulate("paperUrl", examSubject.getPaperUrl() == null ? "" : examSubject.getPaperUrl());
            subject.accumulate("answerUrl", examSubject.getAnswerUrl() == null ? "" : examSubject.getAnswerUrl());
            setting.accumulate("subject", subject);
        }
        return setting;
    }

    private double[] getSplitConfig() {
        String strs[] = splitConfig.split(",");
        double[] config = new double[strs.length];
        for (int i = 0; i < strs.length; i++) {
            config[i] = Double.parseDouble(strs[i]);
        }
        return config;
    }

    private void releaseUser(Integer examId, String subjectCode, Integer userId) {
        try {
            lockService.waitlock(LockType.USER, userId);
            inspectedService.releaseByUserId(examId, subjectCode, userId);
        } catch (Exception e) {
            log.error("release user error", e);
        } finally {
            lockService.unlock(LockType.USER, userId);
        }
    }

    private void releaseStudent(Integer studentId) {
        try {
            lockService.waitlock(LockType.STUDENT, studentId);
            ExamStudent student = studentService.findById(studentId);
            inspectedService.releaseByStudent(student);
        } catch (Exception e) {
            log.error("release user error", e);
        } finally {
            lockService.unlock(LockType.STUDENT, studentId);
        }
    }
}
